<?php
define('LOG_FILE','/var/log/esia/egide_collector.log');
define('LOG_LEVEL',LOG_DEBUG);

$_SERVER['DOCUMENT_ROOT'] = '/usr/local/esia/public_html/';

$path = $_SERVER['DOCUMENT_ROOT'];
include $path.'/esia.conf.php';
include_once USR_CLASS;
include_once LIB_LOG;
include_once MVCMODEL;
include_once $path.'/Model_esia.php';
include_once $path.'/plugin/gesa/model_gesa.php';

write_log(LOG_NOTICE,'START scan collector');

//connection à la base de donnée
write_log(LOG_INFO,'DB connection');
try
{
	$dbconnection = new PDO('pgsql:host='.DB_HOST.';port='.DB_PORT.';dbname='.DB_NAME, DB_USERNAME, DB_PWD);
	$dbconnection->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_WARNING);
}
catch(Exception $e)
{
	$err='Unable to connect to DB. ';
	$err.='PDO SQL ERROR N° : '.$e->getCode();
	$err.='  MESSAGE : '.$e->getMessage();
	write_log(LOG_EMERG,$err);
}

$arr = Model_esia::getWebPlugin($dbconnection,false,'WEBP_EGIDESCAN');
$plugin_url = $arr[0]['apache_path'];
include_once $path.$plugin_url.'model_egidescan.php';


//define('PLUGINS_PATH','/usr/local/esia/plugins');
//define('TMP_DIR',"/tmp/scan");

$scanner = new ScanManager();

try{
	$new_tasks = Model_egidescan::getCurrentScanList($dbconnection,Model_egidescan::PENDING_TASK);
	$old_tasks = Model_egidescan::getCurrentScanList($dbconnection,Model_egidescan::TERMINATED_TASK);
	$pending_tasks = array_merge($new_tasks,$old_tasks);
	$processing_tasks = Model_egidescan::getCurrentScanList($dbconnection,Model_egidescan::PROCESSING_TASK);
}catch(Exception $e)
{
	$err='Unable get task from DB. ';
	$err.='PDO SQL ERROR N° : '.$e->getCode();
	$err.='  MESSAGE : '.$e->getMessage();
	write_log(LOG_ALERT,$err);
}



//Traitement des task pending
write_log(LOG_INFO,'Sending '.count($pending_tasks).' pending tasks');

$new_processing_task=array();
foreach($pending_tasks as $task){
	array_push($new_processing_task,$task['id']);
}

print_r($pending_tasks);

//-A [-i <ID>] -a <ADDR> -e <iface> [-o <OPTS>]
foreach($pending_tasks as $task){
	$id			= $task['id'];
	$agent_id	= $task['agent_id'];

	if( empty($task['options'])) continue;

	$all_node_list = Model_egidescan::getNodesOfGroupList($dbconnection,$task['options']['group_list']);

	$addr_list  = array();
	foreach($all_node_list as $node)
	{
		$ip_addr = $node->getIPAddress();
		if (filter_var($ip_addr, FILTER_VALIDATE_IP)) {
			$addr_list[] = $ip_addr;
		}
	}
	$addr_list = implode(',',$addr_list);
	$iface = $task['iface'];

	$box = Model_gesa::getInstanceGESA($dbconnection,$agent_id);
	$sn = $box->getSerialNumber();

	write_log(LOG_DEBUG,"Sending task $id");
	$arguments = "-A -i $id -e $iface -a $addr_list";

	$result = $scanner->execute($arguments,$sn);

	if( $result['return'] != 0 ){ //Une erreur s'est produite.
		if( preg_match('/ERROR\s*:\s*(.*)/',$result['output'][0],$matches))
		{
			$details=$matches[1];
		}
		else
		{
			$details = $result['output'][0];
		}
		$error_message="Unable to save the task => $details.";
		Model_egidescan::notifyScanningError($dbconnection,$id,$error_message);
		write_log(LOG_WARNING,$error_message);
		echo $error_message."\n";
	}

	try{
		Model_egidescan::updateScanningTaskStatus($dbconnection,$id,Model_egidescan::PROCESSING_TASK);
	}
	catch(Exception $ex){
		$err='Unable to set status processing in DB. ';
		$err.='PDO SQL ERROR N° : '.$e->getCode();
		$err.='  MESSAGE : '.$e->getMessage();
		write_log(LOG_ALERT,$err);
	}
}

//Récupération des task terminées.
write_log(LOG_INFO,'Get terminated tasks list for '.count($processing_tasks).' agents');

$agent_list = array();
foreach($processing_tasks as $row)
{
	if( empty($agent_list[$row['agent_id']]))
	{
		$agent_list[$row['agent_id']] = array();
	}
	$agent_list[$row['agent_id']][] = $row['id'];
}

foreach($agent_list as $agent_id => $db_ids){
	$box = Model_gesa::getInstanceGESA($dbconnection,$agent_id);
	$sn = $box->getSerialNumber();

	write_log(LOG_INFO,"Get terminated tasks status for Agent $sn");
	$arguments = "-L -s terminated,error";
	$result = $scanner->execute($arguments,$sn);

	if( $result['return'] != 0 ){ //Une erreur s'est produite.
		preg_match('/ERROR\s*:\s*(.*)/',$result['output'][0],$matches);
		$details=$matches[1];
		$error_message="Unable to get id list => $details.";
		write_log(LOG_CRIT,$error_message);
		echo $error_message."\n";
		continue;
	}
	else{
		$for_delete_ids = array();
		$for_get_ids = array();

		$dev_ids = trim($result['output'][0]);
		if(!empty($dev_ids)) $dev_ids = explode(',',$dev_ids);
		else $dev_ids = array();

		foreach($dev_ids as $dev_id){
			$for_delete_ids[]=$dev_id;

			if(in_array($dev_id,$db_ids)){
				$for_get_ids[]=$dev_id;
			}
		}
	}

	write_log(LOG_INFO,"Download ".count($for_get_ids)." result");
	if(!empty($for_get_ids)){

		$scan_ids = implode(',',$for_get_ids);
		$arguments = "-G -i $scan_ids";
		$result = $scanner->execute($arguments,$sn);

		if( $result['return'] != 0 ){ //Une erreur s'est produite.
			preg_match('/ERROR\s*:\s*(.*)/',$result['output'][0],$matches);
			$details=$matches[1];
			$error_message="Unable to  => $details.";
			echo $error_message."\n";
			write_log(LOG_ERR,$error_message);
			continue;
		}

		$parsed = @json_decode(trim(implode("\n",$result['output'])),true);
		if($parsed == null){
			$error_message = "Unable to parse received data for $scan_ids";
			echo $error_message."\n";
			write_log(LOG_ERR,$error_message);
			continue;
		}
		else {
			foreach($parsed as $scan_id => $scan_result){
				if($scan_result['status'] == Model_egidescan::ERROR_TASK){
					Model_egidescan::notifyScanningError($dbconnection,$scan_id,$scan_result['message']);
					write_log(LOG_WARNING,$scan_result['message']);
				}
				else if($scan_result['status'] == Model_egidescan::TERMINATED_TASK){
					//$data_path = Model_ScanModule::DATA_PATH.'/'.$scan_id;
					try{
						process_data($dbconnection,$scan_id,$scan_result);
						Model_egidescan::updateScanningTaskStatus($dbconnection,$scan_id,Model_egidescan::TERMINATED_TASK);
						write_log(LOG_DEBUG,"Task $scan_id setted TERMINATED");
					}
					catch(Exception $ex){
						Model_egidescan::notifyScanningError($dbconnection,$scan_id,"Unable to update status for $scan_id : ".$ex->getMessage());
						write_log(LOG_ERR,"Unable save result for ".$scan_id);
						continue;
					}
				}
			}
		}
	}

	write_log(LOG_INFO,"Delete ".count($for_delete_ids)." terminated tasks for agent $sn");
	if(!empty($for_delete_ids)){
		$scan_ids = implode(',',$for_delete_ids);
		$arguments = "-D -i $scan_ids";
		$result = $scanner->execute($arguments,$sn);

		if( $result['return'] != 0 ){ //Une erreur s'est produite.
			preg_match('/ERROR\s*:\s*(.*)/',$result['output'][0],$matches);
			$details=$matches[1];
			$error_message="Unable to delete scans  => $details.";
			write_log(LOG_WARNING,$error_message);
			echo $error_message."\n";
		}
	}
}
write_log(LOG_NOTICE,'END scan collector');


function write_log($priority,$message){
	if($priority <= LOG_LEVEL){
		$date = new DateTimeImmutable();
		$time = $date->format('Y-m-d H:i:s.u');
		//file_put_contents ( LOG_FILE , "$time|$priority|$message\n" ,  FILE_APPEND  );
		echo "$time|$priority|$message\n";
	}
	if( $priority == LOG_EMERG) exit(1);
}

function process_data($dbconnection,$scan_id,$data)
{
	$t0 = $data['start'];
	$t1 = $data['stop'];

	$tmpfname = sys_get_temp_dir(). '/egide_'.$scan_id.$t1.'.xml';
	file_put_contents($tmpfname,$data['result']);

	$t0 = date("Y-m-d H:i:s",$t0);
	$t1 = date("Y-m-d H:i:s",$t1);

	$scan_info = Model_egidescan::getScanById($dbconnection,$scan_id);
	$agent_id = $scan_info['agent_id'];
	Model_egidescan::addScanHistory($dbconnection, $t0,$t1,$scan_id,$data,Model_egidescan::TERMINATED_TASK);

	$group_list = $scan_info['options']['group_list'];
	$all_node_list = Model_egidescan::getNodesOfGroupList($dbconnection,$group_list);

	$list_node = array();
	foreach($all_node_list as $row)
	{
		$list_node[$row->getIPAddress()] = $row;
	}

	sleep(3);

	$reader = XMLReader::open($tmpfname);
	while ($reader->read())
	{
		if($reader->name == 'host')
		{
			$lastscan = $reader->getAttribute('endtime');
			$host = $reader->readOuterXml();
			$dom = new DOMDocument();
			$dom->loadXML($host);
			$address = $dom->getElementsByTagName('address');
			$ipaddress=null;
			//récupération address IP
			foreach ($address as $adr)
			{
				$type= $adr->attributes[1];
				if($type->nodeName =='addrtype' && $type->nodeValue=='ipv4')
				{
					$ipaddress=$adr->attributes[0]->nodeValue;
					break;
				}
			}
			// récupération des ports
			$ports = $dom->getElementsByTagName('port');
			if(array_key_exists($ipaddress,$list_node))
			{
				$node =  $list_node[$ipaddress];
				$oldEgideService =  model_egidescan::getEgideServicesNode($dbconnection,$node->getId());
				foreach ($ports as $port)
				{
					$attr= $port->attributes[0];
					$portnum=null;
					$portType=null;
					if($attr->nodeName == 'protocol')
					{
						$portType=$attr->nodeValue;
					}
					$attr= $port->attributes[1];
					if($attr->nodeName == 'portid')
					{
						$portnum=$attr->nodeValue;
					}
					$service = $port->getElementsByTagName('service')[0];
					$ser =  array();
					foreach($service->attributes as $attr)
					{
						$ser[$attr->nodeName]=$attr->nodeValue;
					}
					if(!array_key_exists('extrainfo',$ser))
					{
						$ser['extrainfo']='';
					}
					$cpe="";
					$cpes = $port->getElementsByTagName('cpe');
					foreach($cpes as $attr)
					{
						$cpe.=$attr->nodeValue."\n";
					}
					$egide_service_id="";
					if(array_key_exists($portnum,$oldEgideService))
					{
						$egide_service_id = $oldEgideService[$portnum]['id'];
						model_egidescan::updateEgideServicesNode($dbconnection,$egide_service_id,$ser['name'],$ser['product'],$ser['version'],$cpe,$ser['extrainfo'],$scan_info['iface_id']);
					}
					else
					{
						echo "INSERT $plugin".$ser['name']." $portnum/$portType\n";
						$egide_service_id = model_egidescan::insertEgideServicesNode($dbconnection,$node->getId(),$ser['name'],$ser['product'],$ser['version'],$portnum,$portType,$cpe,$ser['extrainfo'],$scan_info['iface_id']);
					}
					//Traitement des vulnérabilité
					$scripts = $port->getElementsByTagName('script');
					foreach ($scripts as $script)
					{
						$attr = $script->attributes[0];
						if($attr->nodeValue =='vulscan')
						{
							$attr = $script->attributes[1];
							if($attr->nodeName =='output')
							{
								$text = preg_replace('#^cve.csv\:#','',$attr->nodeValue);
								$text = trim($text);
								$cves =  explode(';', $text);
								$last = array_pop($cves);
								if(!empty($last))
								{
									$cves[]=$last;
								}
								//insert cves
								model_egidescan::insertVulnerabilityDetect($dbconnection,$egide_service_id,'NIST',$cves);
							}
						}
					}
				}
			}
		}
	}
	unlink($tmpfname);
}

class ScanManager{
	private $plugin;

	public function __construct(){
		$this->plugin = new ExecPlugin(1,60);
	}

	public function execute($arguments,$gesa){
		$RETRY = 3;
		$SLEEP = 15;

		for($i=0;$i<$RETRY;$i++)
		{
			$result = $this->plugin->execute('egide_scan_adm.php', $arguments,$gesa);
			if( $result != 3)
			{
				return $result;
			}
			sleep($SLEEP);
		}
		preg_match('/UNKNOWN\s*-\s*(.*)/',$result['output'][0],$matches);
		$result['output'] = array('ERROR : '.$matches[1]);
		return $result;
	}
}

class ExecPlugin{
	const CHECK_GESA='check_gesa';

	private $_timeout;
	private $_priority;

	public function __construct($priority=3,$timeout=3){
		$this->_priority = $priority;
		$this->_timeout = $timeout;
	}

	public function execute($command,$arguments,$gesa){
		$timeout = $this->_timeout;
		$pri = $this->_priority;
		echo PLUGINS_PATH.'/'.self::CHECK_GESA." -G $gesa -C $command -A '$arguments' -t $timeout -p $pri\n";
		exec(PLUGINS_PATH.'/'.self::CHECK_GESA." -G $gesa -C $command -A '$arguments' -t $timeout -p $pri",$output,$returnexec);
		return array("output"=>$output,"return"=>$returnexec);
	}
}

?>
